@@ -1,7 +1,7 @@
 /* -*-pgsql-c-*- */
 /*
  *
- * $Header: /cvsroot/pgpool/pgpool-II/pool_proto_modules.h,v 1.23 2010/08/26 09:23:06 kitagawa Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pool_proto_modules.h,v 1.24.2.2 2011/02/18 11:45:51 kitagawa Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -35,6 +35,7 @@
 #include "pool_session_context.h"
 
 #define SPECIFIED_ERROR 1
+#define POOL_DUMMY_QUERY "DELETE FROM foo WHERE col = 'pgpool: unable to parse the query'"
 #define POOL_ERROR_QUERY "send invalid query from pgpool to abort transaction"
 
 extern char *copy_table;  /* copy table name */
@@ -42,9 +43,6 @@ extern char *copy_schema;  /* copy table name */
 extern char copy_delimiter; /* copy delimiter char */
 extern char *copy_null; /* copy null string */
 
-/* non 0 if "BEGIN" query with extended query protocol received */
-extern int receive_extended_begin;
-
 extern int is_select_pgcatalog;
 extern int is_select_for_update; /* also for SELECT ... INTO */
 extern bool is_parallel_table;
@@ -155,7 +153,6 @@ extern int is_sequence_query(Node *node);
 extern int is_start_transaction_query(Node *node);
 extern int is_commit_query(Node *node);
 extern int is_strict_query(Node *node); /* returns non 0 if this is strict query */
-extern int load_balance_enabled(POOL_CONNECTION_POOL *backend, Node* node, char *sql);
 extern int need_insert_lock(POOL_CONNECTION_POOL *backend, char *query, Node *node);
 extern POOL_STATUS insert_lock(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend, char *query, InsertStmt *node, int lock_kind);
 extern char *parse_copy_data(char *buf, int len, char delimiter, int col_id);
